package com.convenient.android.common.media.scan

import android.content.ContentUris
import android.provider.MediaStore
import android.util.Log
import com.convenient.android.common.config.MyPdfBaseModule
import com.convenient.android.common.extension.toFile
import com.convenient.android.common.media.MediaBean
import com.convenient.android.common.media.basic.Query
import com.convenient.android.common.media.config.MediaQueryConfig
import com.convenient.android.common.media.config.MediaSortType

/**
 * @classname:
 * @author: LiuXiaoLong
 * @date: 2022/8/3
 * description: 通过系统MediaStore 扫描存储中的文件内容
 * MediaStore.query(dir : String, config : MediaQueryConfig) : List<MediaBean>
 */
object MediaStore : Query {

    /**
     * 要查询的数据
     */
    private val projection = arrayOf(
        MediaStore.Files.FileColumns.DATA,
        MediaStore.Files.FileColumns.MIME_TYPE,
        MediaStore.Files.FileColumns.SIZE,
        MediaStore.Files.FileColumns.DATE_MODIFIED,
        MediaStore.Files.FileColumns._ID,
        MediaStore.Files.FileColumns.DISPLAY_NAME
    )


    override fun query(dir: String, config: MediaQueryConfig.() -> Unit): List<MediaBean> {
        return query(dir, MediaQueryConfig().also(config))
    }


    override fun query(dir: String, config: MediaQueryConfig): List<MediaBean> {

        val selectBuilder = StringBuilder()
        val selectionList = mutableListOf<String>()
        //文件夹查询限制
        val dirSelection = getQueryDirSubStatement(dir)
        if (dirSelection.isNotEmpty()) {
            selectionList.add(dirSelection)
        }
        //添加查询的文件格式限制
        val mimeTypesSelection = getMimeTypesSelection(config.supportMimeTypes)
        if (mimeTypesSelection.isNotEmpty()) {
            selectionList.add(mimeTypesSelection)
        }

        selectionList.forEachIndexed { index, s ->
            if (index != selectionList.lastIndex) {
                selectBuilder.append(s).append(" and ")
            } else {
                selectBuilder.append(s)
            }
        }
        val sort = getSortSubStatement(config)

        Log.e("MediaStore", "查询语句:$selectBuilder")
        Log.e("MediaStore", "排序语句:${sort}")

        val cursor = MyPdfBaseModule.getAppContext()?.contentResolver?.query(
            MediaStore.Files.getContentUri("external"),
            projection,
            selectBuilder.toString(),
            null,
            sort
        )

        val list = mutableListOf<MediaBean>()
        cursor?.apply {
            try {
                while (moveToNext()) {
                    val path = getString(getColumnIndexOrThrow(MediaStore.Files.FileColumns.DATA))
                    val mediaFile = path.toFile()
                    if (mediaFile?.isFile?.not() == true){
                        continue
                    }
                    if (config.ignoreChildFiles.any {
                            path.startsWith(it.absolutePath)
                        }) {
                        //如果是忽略的文件或文件夹，直接跳过
                        continue
                    }

                    if (config.recursively.not() && mediaFile?.parent?.equals(dir)?.not() == true){
                        //没有开启递归子目录，并且文件所在的文件夹与扫描的目录不一致，则跳过
                        continue
                    }

                    val id = getLong(getColumnIndexOrThrow(MediaStore.Files.FileColumns._ID))
                    val name = getString(getColumnIndexOrThrow(MediaStore.Files.FileColumns.DISPLAY_NAME))
                    val lastModified = getLong(getColumnIndexOrThrow(MediaStore.Files.FileColumns.DATE_MODIFIED))
                    val contentUri = ContentUris.withAppendedId(MediaStore.Files.getContentUri("external"), id)

                    list.add(
                        MediaBean(
                            mediaPath = path,
                            name = name,
                            extension = mediaFile?.extension ?: "",
                            isFile = true,
                            lastModified = lastModified,
                            parentPath = mediaFile?.parent ?: "",
                            length = mediaFile?.length(),
                            uri = contentUri
                        )
                    )

                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
        }

        return list
    }

    /**
     * 查询指定目录的子语句
     * ( _data LIKE '/storage/emulated/0/DCIM/%' )
     */
    private fun getQueryDirSubStatement(dir: String): String {
        if (dir.isEmpty()) return ""
        val tempDir = if (dir.endsWith("/")) dir else "${dir}/"
        return "( ${MediaStore.Files.FileColumns.DATA} LIKE '${tempDir}%' )"
    }

    /**
     * 查询文件格式限制的子语句
     * (_data LIKE '%pdf')
     */
    private fun getMimeTypesSelection(mimeTypes: List<String>): String {
        if (mimeTypes.isEmpty()) return ""
        val select = StringBuilder()
        mimeTypes.forEachIndexed { index, s ->
            when {
                index == 0 && mimeTypes.size != 1 -> select.append("(${MediaStore.Files.FileColumns.DATA} LIKE '%${s}'")
                index == mimeTypes.lastIndex && mimeTypes.size != 1 -> select.append(" or ${MediaStore.Files.FileColumns.DATA} LIKE '%${s}' )")
                mimeTypes.size == 1 -> select.append("(${MediaStore.Files.FileColumns.DATA} LIKE '%${s}')")
                else -> select.append(" or ${MediaStore.Files.FileColumns.DATA} LIKE '%${s}'")
            }
        }
        return select.toString()
    }

    /**
     * 排序的子语句， 根据名称、时间、大小排序
     * _display_name desc or asc
     * date_modified desc or asc
     * _size desc or asc
     */
    private fun getSortSubStatement(config: MediaQueryConfig): String {

        return when (config.sortType) {
            MediaSortType.NAME -> "${MediaStore.Files.FileColumns.DISPLAY_NAME} ${config.order.name}"
            MediaSortType.DATE -> "${MediaStore.Files.FileColumns.DATE_MODIFIED} ${config.order.name}"
            MediaSortType.SIZE -> "${MediaStore.Files.FileColumns.SIZE} ${config.order.name}"
        }
    }


}